/*
* test-app.cc    : Test Application
* author         : Fabio Silva and Chalermek Intanagonwiwat
*
* Copyright (C) 2000-2001 by the Unversity of Southern California
* $Id: test-app.cc,v 1.2 2002/02/17 07:42:28 johnh Exp $
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License,
* version 2, as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
*
*/

#include "test-app.hh"

/* 
 * This application demonstrates using the timer package
 * by getting two timers going.
 */

/*
* In this example, the return value from expire is used 
* to indicate what should be done with the timer
*  = 0   Add timer again to queue
*  > 0   Set new_timeout as timeout value for timer 
*  < 0   Discard timer 
*/  
int TestTimer1::expire(handle hdl)
{
	return(app_->ProcessTimer(hdl,p_));
}


int TestTimer2::expire(handle hdl)
{
	struct timeval tv;

	getTime(&tv);
	fprintf(stderr, "Timer 2 has expired %d times! Time = %d:%d\n",
		p_, tv.tv_sec, tv.tv_usec);
	fflush(NULL);
	return 0;
}

void TestApp::start()
{
	struct timeval tmv;
	int status;

	// Change while condition to reflect what is required for Project 1
	// ex: Routing table stabalization. 
	while (1) {
		timersManager_->NextTimerTime(&tmv);
		if (tmv.tv_sec == 0) {
			// No Timer have been defined 
			tmv.tv_sec= POLLING_INTERVAL;
			tmv.tv_usec=0;
		}
		/* The select call here will wait for tv seconds before expiring 
		 * You need to  modifiy it to listen to multiple sockets and add code for 
		 * packet processing. Refer to the select man pages or "Unix Network 
		 * Programming" by R. Stevens Pg 156.
		 */
		status = select(0, NULL, NULL, NULL, &tmv);
		
		if (status < 0){
			// This should not happen
			fprintf(stderr, "Select returned %d\n", status);
		}else{
			if (status == 0){
				// Timer expired, Hence process it 
				timersManager_->ExecuteTimer();
			}
			if (status > 0){
				// The socket has received data.
				// Perform packet processing.
		    
			}
		}
	}
	return;
}

int TestApp::ProcessTimer(handle hdl,int p)
{
	struct timeval tv;

	getTime(&tv);
	fprintf(stderr, "Timer 1 has expired %d times! Time = %d:%d\n",
		p, tv.tv_sec, tv.tv_usec);
	fflush(NULL);
	return 0;
}

TestApp::TestApp()
{
	TimerCallbacks *tcb;
	struct timeval tv;

	getTime(&tv);

	fprintf(stderr,"Start Time = %d:%d\n",
		tv.tv_sec,tv.tv_usec);
	// Create the timer event management class
	timersManager_ = new Timers;

	// Create callback classes and set up pointers
	tcb = new TestTimer1(this,1);
	timersManager_->AddTimer(3000, tcb);

	tcb = new TestTimer2(1);
	timersManager_->AddTimer(10000, tcb);
}

int main()
{
	TestApp *app;

	/* Initialize the Timer test application  by
	 * adding timers initially. 
	 *     -Add a timer for each packet sent.
	 *     -Set the timeout value appropriately 
	 */
	app = new TestApp;
	// Start the timers 
	app->start();

	return 0;
}
